package main

import "fmt"

// returns a pair of channels for communicating to the server
func Serv() (as chan chan int, bs chan chan int) {
	// create two channels
	// for sending names of channels for sending integers
	as = make(chan chan int)
	bs = make(chan chan int)
	// launch a goroutine for serving requests
	go func() {
		for {
			// forward messages from as to bs
			c := <-as
			bs <- c
		}
	}()
	return  // naked return
}

// for 10 times 
// creates a channel ch
// sends the channel to the server on as
// sends an integer on ch
func A(as chan chan int) {
	for i := 0; i < 10; i++ {
		ch := make(chan int)
		fmt.Printf("created %v (%T) for sending %v\n", ch, ch, i)
		as <- ch // send ch to the server
		ch <- i  // send i on ch
	}
}

// for 10 times
// receives a channel ch from the server
// receives an integer on ch
func B(bs chan chan int) {
	for i := 0; i < 10; i++ {
		ch := <-bs
		n := <-ch
		fmt.Printf("received %v on %v\n", n, ch)
	}

}

func main() {
	as, bs := Serv() // launch server
	go A(as)         // launch A
	B(bs)            // launch B
}

// ---------------------------------------------------
// using close

// returns a pair of channels for communicating to the server
func Serv() (as chan chan int, bs chan chan int) {
	// create two channels
	// for sending names of channels for sending integers
	as = make(chan chan int)
	bs = make(chan chan int)
	// launch a goroutine for serving requests
	go func() {
		// until as is active
		for c, ok := <-as; ok; c, ok = <-as {
			// forward messages from as to bs
			bs <- c
		}
		// clsoes bs
		close(bs)
	}()
	return // naked return
}

// for 10 times
// creates a channel ch
// sends the channel to the server on as
// sends an integer on ch
func A(as chan chan int) {
	for i := 0; i < 10; i++ {
		ch := make(chan int)
		fmt.Printf("created %v (%T) for sending %v\n", ch, ch, i)
		as <- ch // send ch to the server
		ch <- i  // send i on ch
	}
	// closes the communication with server
	close(as)
}

// for 10 times
// receives a channel ch from the server
// receives an integer on ch
func B(bs chan chan int) {
	// until bs is active
	for ch, ok := <-bs; ok; ch, ok = <-bs {
		n := <-ch
		fmt.Printf("received %v on %v\n", n, ch)
	}
	println("done")

}

func main() {
	as, bs := Serv() // launch server
	go A(as)         // launch A
	B(bs)            // launch B
}
