// A concurrent prime sieve

package main

import "fmt"

// Send the sequence 2, 3, 5, 7, 9, ... to channel ch
// <-chan int: used only for receiving
func Generate() <-chan int {
	out := make(chan int)
	go Gen(out)
	return out
}

func Gen(out chan<- int) {
	out <- 2
	// note the missing guard!
	for i := 3; ; i = i + 2 {
		out <- i
	}	
}

// receives the list of primes and print them
func Rec(in <-chan int) {
	for i := 1; i <= 50; i++ {
		// receives a prime
		prime := <-in
		fmt.Println(prime,"is prime n.",i)
		// install a new filter
		in = Filter(in, prime)
	}
}

// Copy the values from channel 'in' to channel 'out'
// removing those divisible by 'prime'
// <-chan int: used only for receiving
// chan<- int: used only for sending
func Filter(in <-chan int, prime int) <-chan int {
	out := make(chan int)
	go Fil(in,out,prime)
	return out
}

func Fil(in <-chan int, out chan<- int, prime int) {
	fmt.Printf("--- Created filter from %v to %v on prime %v\n", in, out, prime)
	for {
		i := <-in // Receive value from 'in'
		fmt.Printf("--- (filter %v) receives %v on %v\n", prime, i, in)
		if i%prime != 0 {
			fmt.Printf("--- (filter %v) sends %v to %v\n", prime, i, out)
			out <- i // Send 'i' to 'out'
		} else {
			fmt.Printf("--- (filter %v) discards %v\n", prime, i)
		}
	}
}

// The prime sieve: Daisy-chain Filter processes.
func main() {
	ch := Generate() // Launch Generate goroutine.
	Rec(ch)
}
